@@ -206,6 +206,12 @@ public:
     inline bool IsIPv6(void) const {
         return (addr.ss.ss_family == AF_INET6);
     }
+    inline bool IsNAT64(void) const {
+        return (addr.ss.ss_family == AF_INET6 &&
+          addr.in6.sin6_addr.s6_addr32[0] == 0x9bff6400);
+    }
+
+    static bool GetNAT64Addr(const ndAddr &addr_nat64, ndAddr &addr_ipv4);
 
     enum class MakeFlags : uint8_t {
         NONE = 0x0,
@@ -221,8 +227,8 @@ public:
 
     inline const std::string &
     GetString(ndFlags<MakeFlags> flags = MakeFlags::NONE) const {
-        static std::recursive_mutex lock;
-        std::lock_guard<std::recursive_mutex> lg(lock);
+        static std::mutex lock;
+        std::lock_guard<std::mutex> lg(lock);
         if (flags != cached_flags || cached_addr.empty()) {
             cached_flags = flags;
             ndAddr::MakeString(*this, cached_addr, cached_flags);
@@ -526,7 +532,11 @@ public:
     bool operator==(const ndRadixNetworkEntry &rhs) const {
         return prefix_len == rhs.prefix_len && addr == rhs.addr;
     }
-
+#if 0
+    bool operator!=(const ndRadixNetworkEntry &rhs) const {
+        return addr != rhs.addr;
+    }
+#endif
     bool operator<(const ndRadixNetworkEntry &rhs) const {
         if (addr == rhs.addr)
             return prefix_len < rhs.prefix_len;
@@ -673,6 +683,12 @@ public:
     bool RemoveGroup(const std::string &tag);
     bool RemoveGroupAddress(const std::string &tag, const ndAddr &addr);
     bool LookupGroupAddress(const std::string &tag, const ndAddr &addr);
+    bool GetGroupFilename(const std::string &tag, std::string &filename);
+    bool SetGroupFilename(const std::string &tag);
+    bool LoadGroupAddress(const std::string &tag, const std::string &filename);
+    bool SaveGroupAddress(const std::string &tag, const std::string &filename);
+    void LoadGroupAddresses(const std::string &path);
+    void SaveGroupAddresses(const std::string &path);
 
     void Classify(ndAddr::Type &type, const ndAddr &addr);
     inline void
@@ -711,10 +727,11 @@ protected:
         ndAddr, ndAddr, ndAddr::ndAddrHash, ndAddr::ndAddrEqual> neighbor_map;
     neighbor_map neighbors;
 
-    mutable std::mutex lock_groups;
+    mutable std::recursive_mutex lock_groups;
     typedef std::unordered_set<
         ndAddr, ndAddr::ndAddrHash, ndAddr::ndAddrEqual> group_addr_set;
     std::unordered_map<std::string, group_addr_set> groups;
+    std::unordered_map<std::string, std::string> group_filenames;
 };
 
 typedef std::unordered_set<ndAddr, ndAddr::ndAddrHash, ndAddr::ndAddrEqual> ndInterfaceAddrs;
